@@ -39,9 +39,9 @@ CommandScreen::CommandScreen()
 : Command()
 {
 	name = "screen";
-	summary = "Determine whether query sequences are within a larger pool of sequences.";
-	description = "Determine how well query sequences are contained within a pool of sequences. The queries must be formatted as a single Mash sketch file (.msh), created with the `mash sketch` command. The <pool> files can be contigs or reads, in fasta or fastq, gzipped or not, and \"-\" can be given for <pool> to read from standard input. The <pool> sequences are assumed to be nucleotides, and will be 6-frame translated if the <queries> are amino acids. The output fields are [identity, shared-hashes, median-multiplicity, p-value, query-ID, query-comment], where median-multiplicity is computed for shared hashes, based on the number of observations of those hashes within the pool.";
-    argumentString = "<queries>.msh <pool> [<pool>] ...";
+	summary = "Determine whether query sequences are within a larger mixture of sequences.";
+	description = "Determine how well query sequences are contained within a mixture of sequences. The queries must be formatted as a single Mash sketch file (.msh), created with the `mash sketch` command. The <mixture> files can be contigs or reads, in fasta or fastq, gzipped or not, and \"-\" can be given for <mixture> to read from standard input. The <mixture> sequences are assumed to be nucleotides, and will be 6-frame translated if the <queries> are amino acids. The output fields are [identity, shared-hashes, median-multiplicity, p-value, query-ID, query-comment], where median-multiplicity is computed for shared hashes, based on the number of observations of those hashes within the mixture.";
+    argumentString = "<queries>.msh <mixture> [<mixture>] ...";
 	
 	useOption("help");
 	useOption("threads");
@@ -291,7 +291,7 @@ int CommandScreen::run() const
 	
 	for ( unordered_set<MinHashHeap *>::const_iterator i = minHashHeaps.begin(); i != minHashHeaps.end(); i++ )
 	{
-		HashList hashList(parameters.kmerSize);
+		HashList hashList(parameters.use64);
 		
 		(*i)->toHashList(hashList);
 		
@@ -322,7 +322,7 @@ int CommandScreen::run() const
 	*/
 	
 	uint64_t setSize = minHashHeap.estimateSetSize();
-	cerr << "   Estimated distinct" << (trans ? " (translated)" : "") << " k-mers in pool: " << setSize << endl;
+	cerr << "   Estimated distinct" << (trans ? " (translated)" : "") << " k-mers in mixture: " << setSize << endl;
 	
 	if ( setSize == 0 )
 	{
@@ -550,38 +550,6 @@ CommandScreen::HashOutput * hashSequence(CommandScreen::HashInput * input)
 				break;
 			}
 			
-			//kmersTotal++; TODO
-			
-			if ( ! noncanonical )
-			{
-				bool debug = false;
-				useRevComp = true;
-				bool prefixEqual = true;
-	
-				if ( debug ) {for ( uint64_t k = j; k < j + kmerSize; k++ ) { cout << *(seq + k); } cout << endl;}
-				
-				for ( uint64_t k = 0; k < kmerSize; k++ )
-				{
-					char base = seq[j + k];
-					char baseMinus = seqRev[l - j - kmerSize + k];
-		
-					if ( debug ) cout << baseMinus;
-		
-					if ( prefixEqual && baseMinus > base )
-					{
-						useRevComp = false;
-						break;
-					}
-		
-					if ( prefixEqual && baseMinus < base )
-					{
-						prefixEqual = false;
-					}
-				}
-	
-				if ( debug ) cout << endl;
-			}
-	
 			const char * kmer;
 			
 			if ( trans )
@@ -590,14 +558,15 @@ CommandScreen::HashOutput * hashSequence(CommandScreen::HashInput * input)
 			}
 			else
 			{
-				kmer = useRevComp ? seqRev + l - j - kmerSize : seq + j;
+				const char *kmer_fwd = seq + j;
+				const char *kmer_rev = seqRev + length - j - kmerSize;
+				kmer = (noncanonical || memcmp(kmer_fwd, kmer_rev, kmerSize) <= 0) ? kmer_fwd : kmer_rev;
 			}
 			
 			//cout << kmer << '\t' << kmerSize << endl;
 			hash_u hash = getHash(kmer, kmerSize, seed, use64);
 			//cout << kmer << '\t' << hash.hash64 << endl;
 			input->minHashHeap->tryInsert(hash);
-			
 			uint64_t key = use64 ? hash.hash64 : hash.hash32;
 			
 			if ( input->hashCounts.count(key) == 1 )
@@ -637,7 +606,7 @@ double pValueWithin(uint64_t x, uint64_t setSize, double kmerSpace, uint64_t ske
         return 1.;
     }
     
-    double r = 1. / (1. + kmerSpace / setSize);
+    double r = double(setSize) / kmerSpace;
     
 #ifdef USE_BOOST
     return cdf(complement(binomial(sketchSize, r), x - 1));
